@@ -95,6 +95,9 @@ class AiohttpWslinkServer(object):
     def get_port(self):
         return self.app["state"]["runner"].addresses[0][1]
 
+    def get_last_active_client_id(self):
+        return self.app["state"].get("last_active_client_id", None)
+
     async def start(self, port_callback=None):
         app = self.app
         server_config = self.config
@@ -139,6 +142,13 @@ class AiohttpWslinkClient(object):
         self._url = config.get("reverse_url")
         self._server_protocol = config.get("ws_protocol")
         self._ws_handler = WslinkHandler(self._server_protocol, self)
+        self.state = {}
+
+    def __getitem__(self, key):
+        return getattr(self, key)
+
+    def __setitem__(self, key, value):
+        return setattr(self, key, value)
 
     def get_app(self):
         return self
@@ -149,6 +159,9 @@ class AiohttpWslinkClient(object):
     def get_port(self):
         return 0
 
+    def get_last_active_client_id(self):
+        return self.state.get("last_active_client_id", None)
+
     async def start(self, port_callback=None):
         if port_callback is not None:
             port_callback(self.get_port())
@@ -193,7 +206,9 @@ def create_webserver(server_config):
         # Ensure longer path are registered first
         for route in sorted(static_routes.keys(), reverse=True):
             server_path = static_routes[route]
-            routes.append(aiohttp_web.static(_fix_path(route), server_path))
+            routes.append(
+                aiohttp_web.static(_fix_path(route), server_path, append_version=True)
+            )
 
         # Resolve / => index.html
         web_app.router.add_route("GET", "/", _root_handler)
@@ -218,13 +233,13 @@ def create_webserver(server_config):
 
 class WslinkHandler(object):
     def __init__(self, protocol=None, web_app=None):
-        self._valid_token = False
         self.serverProtocol = protocol
         self.web_app = web_app
         self.functionMap = {}
         self.attachmentsReceived = {}
         self.attachmentsRecvQueue = []
         self.connections = {}
+        self.authentified_client_ids = set()
         self.attachment_atomic = asyncio.Lock()
 
         # Build the rpc method dictionary, assuming we were given a serverprotocol
@@ -281,22 +296,24 @@ class WslinkHandler(object):
             aiohttp_app["state"]["shutdown_task"].cancel()
             aiohttp_app["state"]["shutdown_task"] = None
 
-        await current_ws.prepare(request)
+        try:
+            await current_ws.prepare(request)
 
-        await self.onConnect(request, client_id)
+            await self.onConnect(request, client_id)
 
-        async for msg in current_ws:
-            await self.onMessage(msg, client_id)
+            async for msg in current_ws:
+                await self.onMessage(msg, client_id)
+        finally:
+            await self.onClose(client_id)
 
-        await self.onClose(client_id)
+            del self.connections[client_id]
+            self.authentified_client_ids.discard(client_id)
 
-        del self.connections[client_id]
+            logging.info("client {0} disconnected".format(client_id))
 
-        logging.info("client {0} disconnected".format(client_id))
-
-        if not self.connections:
-            logging.info("No more connections, scheduling shutdown")
-            _schedule_shutdown(aiohttp_app)
+            if not self.connections:
+                logging.info("No more connections, scheduling shutdown")
+                _schedule_shutdown(aiohttp_app)
 
         return current_ws
 
@@ -354,7 +371,7 @@ class WslinkHandler(object):
                     and ("secret" in args[0])
                     and await self.validateToken(args[0]["secret"], client_id)
                 ):
-                    self._valid_token = True
+                    self.authentified_client_ids.add(client_id)
                     await self.sendWrappedMessage(
                         rpcid,
                         {"clientID": "c{0}".format(client_id)},
@@ -382,7 +399,7 @@ class WslinkHandler(object):
         payload = msg.data
 
         if isBinary:
-            if self._valid_token:
+            if self.isClientAuthenticated(client_id):
                 # assume all binary messages are attachments
                 try:
                     key = self.attachmentsRecvQueue.pop(0)
@@ -426,7 +443,7 @@ class WslinkHandler(object):
             return
 
         # Prevent any further processing if token is not valid
-        if not self._valid_token:
+        if not self.isClientAuthenticated(client_id):
             await self.sendWrappedError(
                 rpcid,
                 pub.AUTHENTICATION_ERROR,
@@ -472,6 +489,7 @@ class WslinkHandler(object):
             args.insert(0, obj)
 
             try:
+                self.web_app["state"]["last_active_client_id"] = client_id
                 results = func(*args, **kwargs)
                 if inspect.isawaitable(results):
                     results = await results
@@ -539,7 +557,33 @@ class WslinkHandler(object):
             return True
         return token == self.serverProtocol.secret
 
-    async def sendWrappedMessage(self, rpcid, content, method="", client_id=None):
+    def isClientAuthenticated(self, client_id):
+        return client_id in self.authentified_client_ids
+
+    def getAuthenticatedWebsockets(self, client_id=None, skip_last_active_client=False):
+        if skip_last_active_client:
+            last_c = self.web_app["state"].get("last_active_client_id")
+            return [
+                self.connections[c]
+                for c in self.connections
+                if self.isClientAuthenticated(c) and c != last_c
+            ]
+
+        if client_id:
+            if self.isClientAuthenticated(client_id):
+                return [self.connections.get(client_id)]
+            else:
+                return []
+
+        return [
+            self.connections[c]
+            for c in self.connections
+            if self.isClientAuthenticated(c)
+        ]
+
+    async def sendWrappedMessage(
+        self, rpcid, content, method="", client_id=None, skip_last_active_client=False
+    ):
         wrapper = {
             "wslink": "1.0",
             "id": rpcid,
@@ -559,11 +603,7 @@ class WslinkHandler(object):
             )
             return
 
-        websockets = (
-            [self.connections.get(client_id)]
-            if client_id
-            else [self.connections[c] for c in self.connections]
-        )
+        websockets = self.getAuthenticatedWebsockets(client_id, skip_last_active_client)
 
         # Check if any attachments in the map go with this message
         attachments = pub.publishManager.getAttachmentMap()
@@ -628,10 +668,16 @@ class WslinkHandler(object):
             if ws is not None:
                 await ws.send_str(encMsg)
 
-    def publish(self, topic, data, client_id=None):
+    def publish(self, topic, data, client_id=None, skip_last_active_client=False):
         client_list = [client_id] if client_id else [c_id for c_id in self.connections]
         for client in client_list:
-            pub.publishManager.publish(topic, data, client_id=client)
+            if self.isClientAuthenticated(client):
+                pub.publishManager.publish(
+                    topic,
+                    data,
+                    client_id=client,
+                    skip_last_active_client=skip_last_active_client,
+                )
 
     def addAttachment(self, payload):
         return pub.publishManager.addAttachment(payload)
