@@ -241,7 +241,7 @@ def start_webserver(
 
         # Confifugre SSL
         if len(options.ssl) > 0:
-            from .ssl_context import generate_ssl_pair
+            from .ssl_context import generate_ssl_pair, ssl
 
             if options.ssl == "adhoc":
                 options.ssl = generate_ssl_pair(server_config["host"])
@@ -249,7 +249,8 @@ def start_webserver(
                 tokens = options.ssl.split(",")
                 if len(tokens) != 2:
                     raise Exception(
-                        f"ssl configure must be \"adhoc\" or a tuple of files \"cert,key\"")
+                        f'ssl configure must be "adhoc" or a tuple of files "cert,key"'
+                    )
                 options.ssl = tokens
             cert, key = options.ssl
             context = ssl.SSLContext(ssl.PROTOCOL_TLS_SERVER)
